/////////////////////////////////////////////////////////////////////////////////
// MAIN.C - DStar - A unique TI puzzle game originally created by A Von Dollen
// C version created by Henk Poley  BlackBerry port by Athlor (athlor@gmail.com)
/////////////////////////////////////////////////////////////////////////////////
#include <pager.h>
#include <ribbon.h>
#include "dstar.h"

char VersionPtr[] = "DStar";
int AppStackSize = 3000;

void fadeout(void)
{
  for(i=c; i>0; i--){
    LcdSetContrast(i);
    RimSleep(10);
  }
}

void fadein(void)
{
  for(i=0; i<c+1; i++){
    LcdSetContrast(i);
    RimSleep(10);
  }
}

void SetupLevel(void)
{
  int x;
  char *ptr, *ptr2;

  PieceIsBall=FALSE; // Fresh level, so start with the ball

  ptr2 = Board;            // We copy to the Board
  ptr = levels+(Level*38); // from the Level data

  BallOffset = *ptr++;       // First two bytes are the ball
  BoxOffset = *ptr++;        // and the box position

  for (x=0; x!=36; x++) { // Decompress Level into the Board
    *ptr2++ = ((*ptr) >> 6)&3;
    *ptr2++ = ((*ptr) >> 4)&3;
    *ptr2++ = ((*ptr) >> 2)&3;
    *ptr2++ = (*ptr) & 3;
    ptr++;
  }
  *(Board+BallOffset)=BALL; // Put the ball and box
  *(Board+BoxOffset)=BOX;   // into their Board position

  DrawBoard(); // Display clean Board
}

void DrawBoard(void)
{
  int x, y;
  char *ptr;

  ptr = Board;
  LcdClearDisplay(); // Clear screen
  LcdDrawBox(DRAW_BLACK,48,6,110,25); // Draw title frame
  RimSprintf(t,31,"DStar v1.0\nLevel: %d",Level);
  LcdPutStringXY(-1,8,t,-1,LCD_CENTERED|LCD_MULTILINE); // Title


  for (y=0; y!=9; y++)
    for (x=0; x!=16; x++)
      LcdRasterOp(COPY_SRC,10,10,&sprites,10*(*ptr++),0,NULL,x*10,y*10+32);
  LcdPutStringXY(64,144,"Mode:",-1,0);
}

// Check if a Level is (not) finished:
// There are 144 squares in each Level
// Note the use of != instead of < or <=
//  - this is faster to execute on the Z80!
char CheckNotFinished(void)
{
  char *ptr = Board;

  for (i=1; i!=144; i++) {
    if (*ptr++ == BUBB) return (TRUE); // Are there any bubbles?
  }
  if (++Level==MAXLEVEL) return (FALSE); // All levels done?

  AlertConfiguration VIB = {
   50,                 // Duty
   50,                 // Volume (initial)
   50,                 // Volume (max)
   1,                  // Repetitions
   3,                  // VibrateTime (1/10 sec)
   CONTINUOUS_VIBRATE, // VibrateType
   VIBRATE_AND_TONE,   // InHolsterNotify
   VIBRATE_AND_TONE,   // OutOfHolsterNotify
   0};                 // Pause
  RimTestAlert(TONE_SEQUENCE_4,&VIB);
  fadeout();
  SetupLevel();  // If not => Next Level!
  fadein();
  return (TRUE); // And keep scanning keys
}

// Check to see if we're running into anything:
// - The box stops for everything (except empty space [= 0])
// - The ball stops for everything except a bubble
char TestNextPosIsStop(char nextpos)
{
  if (!PieceIsBall)
    if (nextpos==BUBB) return (FALSE);
  return (nextpos);
}

void MovePiece(char *ptr, char plusx, char plusy)
{
  char *locn;
  char temp, temp2;
  int x, y;

  temp = PieceIsBall+3;
  temp2 = (plusx+(plusy*16));

  while (1) { // Loop
    locn = *(ptr)+Board;
    if (TestNextPosIsStop(*(locn+temp2))) return; // Till edge

    y = (*(ptr)/16);
    x = (*(ptr)-(y*16))*10;
    y *= 10;

    if (*(locn+temp2)==BUBB) {
      LcdRasterOp(COPY_SRC,10,10,&sprites,10*BUBB,0,NULL,10*plusx+x,10*plusy+y+32);
      RimSpeakerBeep(880,50,50,50);
    }

    *(locn+temp2) = *locn;
    *locn = 0;

    LcdRasterOp(COPY_SRC,10,10,&sprites,0,0,NULL,x,y+32); // Remove old

    LcdRasterOp(COPY_SRC,10,10,&sprites,10*temp,0,NULL,10*plusx+x,10*plusy+y+32); // Set new
    (*ptr) += temp2;
    RimSleep(10);
  }
}

void DStarThread(void)
{
  char *charptr;
  MESSAGE msg;
  PID pid;
  pid.Name = VersionPtr;
  pid.EnableForeground = TRUE;
  pid.Icon = NULL;
  RimSetPID(&pid);
  c=LcdGetContrast(); // Save screen contrast
  RimRequestForeground(RimGetCurrentTaskID());

  Level=(STARTLEV-1);
  SetupLevel(); // Display the first level
  fadein();

  for (;;) // Start message loop
  { // Set up a pointer to the variable we want to change
    // (either the box or the ball)
    charptr = PieceIsBall ? &BoxOffset : &BallOffset;
    CheckNotFinished();
    LcdRasterOp(COPY_SRC,10,10,&sprites,30+PieceIsBall*10,0,NULL,90,143);
    RimGetMessage(&msg);

    switch (msg.Device) {
    case DEVICE_KEYPAD:
      switch (msg.Event) {
      case KEY_DOWN:
      {
        switch (msg.SubMsg) {
        case KEY_ESCAPE: // ESC key exits
          RibbonShowRibbon();
          RimTerminateThread();
          break;
        case 'z': MovePiece(charptr, 0, +1); break; // DOWN
        case 'q': MovePiece(charptr, 0, -1); break; // UP
        case 's': MovePiece(charptr, +1, 0); break; // RIGHT
        case 'a': MovePiece(charptr, -1, 0); break; // LEFT
        case ' ': PieceIsBall ^= 1; break;          // Ball/box toggle
        case 'p': if (++Level==MAXLEVEL) {--Level;break;} // CHEAT (Next level)
          SetupLevel();
          break;
        case 'o': if (--Level == -1) {++Level;break;} // Prev level (fall)
        case 'c': SetupLevel();
        }
        break;
      }
      }
    }
  } // Message loop
}

void PagerMain(void)
{
  RibbonRegisterFunction(VersionPtr,&IconBMP,0,40,DStarThread,AppStackSize);
}
